@@ -162,6 +162,12 @@ glade_app_finalize (GObject * app)
 static void
 build_package_paths (void)
 {
+  const gchar *path;
+
+  path = g_getenv (GLADE_ENV_PIXMAP_DIR);
+  if (path)
+    pixmaps_dir = g_strdup (path);
+
 #if defined (G_OS_WIN32) || (defined (MAC_INTEGRATION) && defined (MAC_BUNDLE))
   gchar *prefix;
 
@@ -173,7 +179,9 @@ build_package_paths (void)
 
 # endif
 
-  pixmaps_dir = g_build_filename (prefix, "share", PACKAGE, "pixmaps", NULL);
+  if (!pixmaps_dir)
+    pixmaps_dir = g_build_filename (prefix, "share", PACKAGE, "pixmaps", NULL);
+
   catalogs_dir = g_build_filename (prefix, "share", PACKAGE, "catalogs", NULL);
   modules_dir = g_build_filename (prefix, "lib", PACKAGE, "modules", NULL);
   locale_dir = g_build_filename (prefix, "share", "locale", NULL);
@@ -184,7 +192,9 @@ build_package_paths (void)
 #else
   catalogs_dir = g_strdup (GLADE_CATALOGSDIR);
   modules_dir = g_strdup (GLADE_MODULESDIR);
-  pixmaps_dir = g_strdup (GLADE_PIXMAPSDIR);
+
+  if (!pixmaps_dir)
+    pixmaps_dir = g_strdup (GLADE_PIXMAPSDIR);
   locale_dir = g_strdup (GLADE_LOCALEDIR);
   bin_dir = g_strdup (GLADE_BINDIR);
   lib_dir = g_strdup (GLADE_LIBDIR);
@@ -199,8 +209,6 @@ glade_init_check (void)
     return;
 
   /* Make sure path accessors work on osx */
-  g_type_init ();
-
   build_package_paths ();
 
   bindtextdomain (GETTEXT_PACKAGE, locale_dir);
@@ -277,6 +285,38 @@ pointer_mode_register_icon (GtkIconFactory *factory,
     }
 }
 
+static void
+register_stock_icon (GtkIconFactory *factory,
+                     const gchar *stock_id,
+                     const gchar *icon_name,
+                     const gchar *file_name,
+                     GtkIconSize size)
+{
+  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+  GdkPixbuf *pixbuf;
+  GtkIconInfo *info;
+  gint w, h;  
+
+  if (gtk_icon_size_lookup (size, &w, &h) &&
+      (info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, MIN (w, h), 0)))
+    {
+      pixbuf = gtk_icon_info_load_icon (info, NULL);
+    }
+  else
+    {
+      gchar *path = g_build_filename (glade_app_get_pixmaps_dir (), file_name, NULL);
+      pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+      g_free (path);
+    }
+
+  if (pixbuf)
+    {
+      GtkIconSet *icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
+      gtk_icon_factory_add (factory, stock_id, icon_set);
+      g_object_unref (pixbuf);
+    }
+}
+
 /*
  * glade_app_register_stock_icons:
  * @size: icon size
@@ -295,6 +335,11 @@ glade_app_register_stock_icons (GtkIconSize size)
   pointer_mode_register_icon (factory, "glade-margin-edit", GLADE_POINTER_MARGIN_EDIT, size);
   pointer_mode_register_icon (factory, "glade-align-edit", GLADE_POINTER_ALIGN_EDIT, size);
 
+  register_stock_icon (factory, "glade-devhelp",
+                       GLADE_DEVHELP_ICON_NAME,
+                       GLADE_DEVHELP_FALLBACK_ICON_FILE,
+                       size);
+  
   gtk_icon_factory_add_default (factory);
 }
 
@@ -315,7 +360,7 @@ glade_init (void)
 
   /* Register icons needed by the UI */
   glade_app_register_stock_icons (GTK_ICON_SIZE_LARGE_TOOLBAR);
-
+  
   init = TRUE;
 }
 
