@@ -564,25 +564,3 @@ glade_standard_boolean_spec (void)
   return g_param_spec_boolean ("boolean", "Boolean",
                                "A boolean value", FALSE, G_PARAM_READWRITE);
 }
-
-GType
-glade_item_appearance_get_type (void)
-{
-  static GType etype = 0;
-
-  if (etype == 0)
-    {
-      static const GEnumValue values[] = {
-        {GLADE_ITEM_ICON_AND_LABEL, "GLADE_ITEM_ICON_AND_LABEL",
-         "icon-and-label"},
-        {GLADE_ITEM_ICON_ONLY, "GLADE_ITEM_ICON_ONLY", "icon-only"},
-        {GLADE_ITEM_LABEL_ONLY, "GLADE_ITEM_LABEL_ONLY", "label-only"},
-        {0, NULL, NULL}
-      };
-
-      etype = g_enum_register_static ("GladeItemAppearance", values);
-
-    }
-
-  return etype;
-}
