@@ -352,6 +352,183 @@ glade_command_check_group (GladeCommand *cmd)
     }
 }
 
+
+/****************************************************/
+/*******  GLADE_COMMAND_PROPERTY_ENABLED      *******/
+/****************************************************/
+typedef struct
+{
+  GladeCommand parent;
+  GladeProperty *property;
+  gboolean old_enabled;
+  gboolean new_enabled;
+} GladeCommandPropertyEnabled;
+
+/* standard macros */
+GLADE_MAKE_COMMAND (GladeCommandPropertyEnabled, glade_command_property_enabled);
+#define GLADE_COMMAND_PROPERTY_ENABLED_TYPE       (glade_command_property_enabled_get_type ())
+#define GLADE_COMMAND_PROPERTY_ENABLED(o)	  (G_TYPE_CHECK_INSTANCE_CAST \
+						   ((o), GLADE_COMMAND_PROPERTY_ENABLED_TYPE, \
+						    GladeCommandPropertyEnabled))
+#define GLADE_COMMAND_PROPERTY_ENABLED_CLASS(k)	  (G_TYPE_CHECK_CLASS_CAST \
+						   ((k), GLADE_COMMAND_PROPERTY_ENABLED_TYPE, \
+						    GladeCommandPropertyEnabledClass))
+#define GLADE_IS_COMMAND_PROPERTY_ENABLED(o)	   (G_TYPE_CHECK_INSTANCE_TYPE ((o), GLADE_COMMAND_PROPERTY_ENABLED_TYPE))
+#define GLADE_IS_COMMAND_PROPERTY_ENABLED_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GLADE_COMMAND_PROPERTY_ENABLED_TYPE))
+
+
+static gboolean
+glade_command_property_enabled_execute (GladeCommand *cmd)
+{
+  GladeCommandPropertyEnabled *me = GLADE_COMMAND_PROPERTY_ENABLED (cmd);
+
+  glade_property_set_enabled (me->property, me->new_enabled);
+
+  return TRUE;
+}
+
+static gboolean
+glade_command_property_enabled_undo (GladeCommand *cmd)
+{
+  GladeCommandPropertyEnabled *me = GLADE_COMMAND_PROPERTY_ENABLED (cmd);
+
+  glade_property_set_enabled (me->property, me->old_enabled);
+
+  return TRUE;
+}
+
+static void
+glade_command_property_enabled_finalize (GObject *obj)
+{
+  GladeCommandPropertyEnabled *me;
+
+  g_return_if_fail (GLADE_IS_COMMAND_PROPERTY_ENABLED (obj));
+
+  me = GLADE_COMMAND_PROPERTY_ENABLED (obj);
+
+  g_object_unref (me->property);
+  glade_command_finalize (obj);
+}
+
+static gboolean
+glade_command_property_enabled_unifies (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandPropertyEnabled *cmd1;
+  GladeCommandPropertyEnabled *cmd2;
+
+  if (!other_cmd)
+    {
+      if (GLADE_IS_COMMAND_PROPERTY_ENABLED (this_cmd))
+        {
+          cmd1 = (GladeCommandPropertyEnabled *) this_cmd;
+
+	  return (cmd1->old_enabled == cmd1->new_enabled);
+        }
+      return FALSE;
+    }
+
+  if (GLADE_IS_COMMAND_PROPERTY_ENABLED (this_cmd) &&
+      GLADE_IS_COMMAND_PROPERTY_ENABLED (other_cmd))
+    {
+      cmd1 = GLADE_COMMAND_PROPERTY_ENABLED (this_cmd);
+      cmd2 = GLADE_COMMAND_PROPERTY_ENABLED (other_cmd);
+
+      return (cmd1->property == cmd2->property);
+    }
+
+  return FALSE;
+}
+
+static void
+glade_command_property_enabled_collapse (GladeCommand *this_cmd,
+					 GladeCommand *other_cmd)
+{
+  GladeCommandPropertyEnabled *this = GLADE_COMMAND_PROPERTY_ENABLED (this_cmd);
+  GladeCommandPropertyEnabled *other = GLADE_COMMAND_PROPERTY_ENABLED (other_cmd);
+  GladeWidget *widget;
+  GladePropertyClass *pclass;
+
+  this->new_enabled = other->new_enabled;
+
+  widget = glade_property_get_widget (this->property);
+  pclass = glade_property_get_class (this->property);
+
+  g_free (this_cmd->priv->description);
+  if (this->new_enabled)
+    this_cmd->priv->description =
+      g_strdup_printf (_("Enabling property %s on widget %s"),
+		       glade_property_class_get_name (pclass),
+		       glade_widget_get_name (widget));
+  else
+    this_cmd->priv->description =
+      g_strdup_printf (_("Disabling property %s on widget %s"),
+		       glade_property_class_get_name (pclass),
+		       glade_widget_get_name (widget));
+}
+
+/**
+ * glade_command_set_property_enabled:
+ * @property: An optional #GladeProperty
+ * @enabled: Whether the property should be enabled
+ *
+ * Enables or disables @property.
+ *
+ * @property must be an optional property.
+ */
+void
+glade_command_set_property_enabled (GladeProperty *property,
+				    gboolean       enabled)
+{
+  GladeCommandPropertyEnabled *me;
+  GladeCommand *cmd;
+  GladeWidget *widget;
+  GladePropertyClass *pclass;
+  gboolean old_enabled;
+
+  /* Sanity checks */
+  g_return_if_fail (GLADE_IS_PROPERTY (property));
+
+  widget = glade_property_get_widget (property);
+  g_return_if_fail (GLADE_IS_WIDGET (widget));
+
+  /* Only applies to optional properties */
+  pclass = glade_property_get_class (property);
+  g_return_if_fail (glade_property_class_optional (pclass));
+
+  /* Fetch current state */
+  old_enabled = glade_property_get_enabled (property);
+
+  /* Avoid useless command */
+  if (old_enabled == enabled)
+    return;
+
+  me = g_object_new (GLADE_COMMAND_PROPERTY_ENABLED_TYPE, NULL);
+  cmd = GLADE_COMMAND (me);
+  cmd->priv->project = glade_widget_get_project (widget);
+
+  me->property = g_object_ref (property);
+  me->new_enabled = enabled;
+  me->old_enabled = old_enabled;
+
+  if (enabled)
+    cmd->priv->description =
+      g_strdup_printf (_("Enabling property %s on widget %s"),
+		       glade_property_class_get_name (pclass),
+		       glade_widget_get_name (widget));
+  else
+    cmd->priv->description =
+      g_strdup_printf (_("Disabling property %s on widget %s"),
+		       glade_property_class_get_name (pclass),
+		       glade_widget_get_name (widget));
+
+  glade_command_check_group (GLADE_COMMAND (me));
+
+  if (glade_command_property_enabled_execute (GLADE_COMMAND (me)))
+    glade_project_push_undo (cmd->priv->project, cmd);
+  else
+    g_object_unref (G_OBJECT (me));
+}
+
 /**************************************************/
 /*******     GLADE_COMMAND_SET_PROPERTY     *******/
 /**************************************************/
@@ -860,6 +1037,17 @@ glade_command_set_name_unifies (GladeCommand *this_cmd, GladeCommand *other_cmd)
   GladeCommandSetName *cmd1;
   GladeCommandSetName *cmd2;
 
+  if (!other_cmd)
+    {
+      if (GLADE_IS_COMMAND_SET_NAME (this_cmd))
+        {
+          cmd1 = (GladeCommandSetName *) this_cmd;
+
+	  return (g_strcmp0 (cmd1->old_name, cmd1->name) == 0);
+        }
+      return FALSE;
+    }
+
   if (GLADE_IS_COMMAND_SET_NAME (this_cmd) &&
       GLADE_IS_COMMAND_SET_NAME (other_cmd))
     {
@@ -1186,10 +1374,6 @@ glade_command_remove (GList *widgets)
 
   g_return_if_fail (widgets != NULL);
 
-  me = g_object_new (GLADE_COMMAND_ADD_REMOVE_TYPE, NULL);
-  me->add = FALSE;
-  me->from_clipboard = FALSE;
-
   /* internal children cannot be deleted. Notify the user. */
   for (list = widgets; list && list->data; list = list->next)
     {
@@ -1216,6 +1400,10 @@ glade_command_remove (GList *widgets)
         }
     }
 
+  me = g_object_new (GLADE_COMMAND_ADD_REMOVE_TYPE, NULL);
+  me->add = FALSE;
+  me->from_clipboard = FALSE;
+
   GLADE_COMMAND (me)->priv->project = glade_widget_get_project (widget);
   GLADE_COMMAND (me)->priv->description = g_strdup ("dummy");
 
@@ -1237,6 +1425,10 @@ glade_command_remove (GList *widgets)
            get_all_parentless_reffed_widgets (cdata->reffed, widget)) != NULL)
         g_list_foreach (cdata->reffed, (GFunc) g_object_ref, NULL);
 
+      /* If we're removing the template widget, then we need to unset it as template */
+      if (glade_project_get_template (GLADE_COMMAND (me)->priv->project) == widget)
+	glade_command_set_project_template (GLADE_COMMAND (me)->priv->project, NULL);
+
       /* Undoably unset any object properties that may point to the removed object */
       glade_command_delete_prop_refs (widget);
 
@@ -2302,3 +2494,483 @@ glade_command_unlock_widget (GladeWidget *widget)
     g_object_unref (G_OBJECT (me));
 
 }
+
+
+/******************************************************************************
+ * 
+ * This command sets the target version of a GladeProject
+ * 
+ *****************************************************************************/
+typedef struct
+{
+  GladeCommand parent;
+  gchar       *catalog;
+  gint         old_major;
+  gint         old_minor;
+  gint         new_major;
+  gint         new_minor;
+} GladeCommandTarget;
+
+GLADE_MAKE_COMMAND (GladeCommandTarget, glade_command_target);
+#define GLADE_COMMAND_TARGET_TYPE	 (glade_command_target_get_type ())
+#define GLADE_COMMAND_TARGET(o)	  	 (G_TYPE_CHECK_INSTANCE_CAST ((o), GLADE_COMMAND_TARGET_TYPE, GladeCommandTarget))
+#define GLADE_COMMAND_TARGET_CLASS(k)	 (G_TYPE_CHECK_CLASS_CAST ((k), GLADE_COMMAND_TARGET_TYPE, GladeCommandTargetClass))
+#define GLADE_IS_COMMAND_TARGET(o)	 (G_TYPE_CHECK_INSTANCE_TYPE ((o), GLADE_COMMAND_TARGET_TYPE))
+#define GLADE_IS_COMMAND_TARGET_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GLADE_COMMAND_TARGET_TYPE))
+
+static gboolean
+glade_command_target_execute (GladeCommand *cmd)
+{
+  GladeCommandTarget *me = (GladeCommandTarget *) cmd;
+
+  glade_project_set_target_version (cmd->priv->project,
+				    me->catalog,
+				    me->new_major,
+				    me->new_minor);
+
+  return TRUE;
+}
+
+static gboolean
+glade_command_target_undo (GladeCommand *cmd)
+{
+  GladeCommandTarget *me = (GladeCommandTarget *) cmd;
+
+  glade_project_set_target_version (cmd->priv->project,
+				    me->catalog,
+				    me->old_major,
+				    me->old_minor);
+
+  return TRUE;
+}
+
+static void
+glade_command_target_finalize (GObject *obj)
+{
+  GladeCommandTarget *me = (GladeCommandTarget *) obj;
+
+  g_free (me->catalog);
+
+  glade_command_finalize (obj);
+}
+
+static gboolean
+glade_command_target_unifies (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandTarget *me;
+
+  /* Do we unify with self ? */
+  if (!other_cmd)
+    {
+      if (GLADE_IS_COMMAND_TARGET (this_cmd))
+        {
+          me = (GladeCommandTarget *) this_cmd;
+
+	  return (me->old_major == me->new_major &&
+		  me->old_minor == me->new_minor);
+        }
+      return FALSE;
+    }
+
+  if (GLADE_IS_COMMAND_TARGET (this_cmd) &&
+      GLADE_IS_COMMAND_TARGET (other_cmd))
+    {
+      GladeCommandTarget *other;
+
+      me = (GladeCommandTarget *) this_cmd;
+      other = (GladeCommandTarget *) other_cmd;
+
+      return g_strcmp0 (me->catalog, other->catalog) == 0;
+    }
+
+  return FALSE;
+}
+
+static void
+glade_command_target_collapse (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandTarget *this;
+  GladeCommandTarget *other;
+
+  g_return_if_fail (GLADE_IS_COMMAND_TARGET (this_cmd) &&
+                    GLADE_IS_COMMAND_TARGET (other_cmd));
+
+  this = GLADE_COMMAND_TARGET (this_cmd);
+  other = GLADE_COMMAND_TARGET (other_cmd);
+
+  this->new_major = other->new_major;
+  this->new_minor = other->new_minor;
+
+  g_free (this_cmd->priv->description);
+  this_cmd->priv->description =
+    g_strdup_printf (_("Setting target version of '%s' to %d.%d"), 
+		     this->catalog, this->new_major, this->new_minor);
+
+}
+
+/**
+ * glade_command_set_project_target:
+ * @project: A #GladeProject
+ * @catalog: The name of the catalog to set the project's target for
+ * @major: The new major version of @catalog to target
+ * @minor: The new minor version of @catalog to target
+ *
+ * Sets the target of @catalog to @major.@minor in @project.
+ */
+void
+glade_command_set_project_target  (GladeProject *project,
+				   const gchar  *catalog,
+				   gint          major,
+				   gint          minor)
+{
+  GladeCommandTarget *me;
+  gint old_major = 0;
+  gint old_minor = 0;
+
+  g_return_if_fail (GLADE_IS_PROJECT (project));
+  g_return_if_fail (catalog && catalog[0]);
+  g_return_if_fail (major >= 0);
+  g_return_if_fail (minor >= 0);
+
+  /* load up the command */
+  me = g_object_new (GLADE_COMMAND_TARGET_TYPE, NULL);
+  GLADE_COMMAND (me)->priv->project = project;
+
+  me->catalog = g_strdup (catalog);
+
+  glade_project_get_target_version (project, me->catalog, &old_major, &old_minor);
+
+  me->new_major = major;
+  me->new_minor = minor;
+  me->old_major = old_major;
+  me->old_minor = old_minor;
+
+  GLADE_COMMAND (me)->priv->description =
+    g_strdup_printf (_("Setting target version of '%s' to %d.%d"), 
+		     me->catalog, me->new_major, me->new_minor);
+
+  glade_command_check_group (GLADE_COMMAND (me));
+
+  /* execute the command and push it on the stack if successful 
+   * this sets the actual policy
+   */
+  if (glade_command_target_execute (GLADE_COMMAND (me)))
+    glade_project_push_undo (GLADE_COMMAND (me)->priv->project, GLADE_COMMAND (me));
+  else
+    g_object_unref (G_OBJECT (me));
+}
+
+/******************************************************************************
+ * 
+ * This command sets the translation domain of a GladeProject
+ * 
+ *****************************************************************************/
+typedef struct
+{
+  GladeCommand parent;
+  gchar       *new_domain;
+  gchar       *old_domain;
+} GladeCommandDomain;
+
+GLADE_MAKE_COMMAND (GladeCommandDomain, glade_command_domain);
+#define GLADE_COMMAND_DOMAIN_TYPE	 (glade_command_domain_get_type ())
+#define GLADE_COMMAND_DOMAIN(o)	  	 (G_TYPE_CHECK_INSTANCE_CAST ((o), GLADE_COMMAND_DOMAIN_TYPE, GladeCommandDomain))
+#define GLADE_COMMAND_DOMAIN_CLASS(k)	 (G_TYPE_CHECK_CLASS_CAST ((k), GLADE_COMMAND_DOMAIN_TYPE, GladeCommandDomainClass))
+#define GLADE_IS_COMMAND_DOMAIN(o)	 (G_TYPE_CHECK_INSTANCE_TYPE ((o), GLADE_COMMAND_DOMAIN_TYPE))
+#define GLADE_IS_COMMAND_DOMAIN_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GLADE_COMMAND_DOMAIN_TYPE))
+
+static gboolean
+glade_command_domain_execute (GladeCommand *cmd)
+{
+  GladeCommandDomain *me = (GladeCommandDomain *) cmd;
+
+  glade_project_set_translation_domain (cmd->priv->project, me->new_domain);
+
+  return TRUE;
+}
+
+static gboolean
+glade_command_domain_undo (GladeCommand *cmd)
+{
+  GladeCommandDomain *me = (GladeCommandDomain *) cmd;
+
+  glade_project_set_translation_domain (cmd->priv->project, me->old_domain);
+
+  return TRUE;
+}
+
+static void
+glade_command_domain_finalize (GObject *obj)
+{
+  GladeCommandDomain *me = (GladeCommandDomain *) obj;
+
+  g_free (me->new_domain);
+  g_free (me->old_domain);
+
+  glade_command_finalize (obj);
+}
+
+static gboolean
+glade_command_domain_unifies (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandDomain *me;
+
+  /* Do we unify with self ? */
+  if (!other_cmd)
+    {
+      if (GLADE_IS_COMMAND_DOMAIN (this_cmd))
+        {
+          me = (GladeCommandDomain *) this_cmd;
+
+	  return g_strcmp0 (me->new_domain, me->old_domain) == 0;
+        }
+      return FALSE;
+    }
+
+  if (GLADE_IS_COMMAND_DOMAIN (this_cmd) &&
+      GLADE_IS_COMMAND_DOMAIN (other_cmd))
+    {
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+glade_command_domain_collapse (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandDomain *this;
+  GladeCommandDomain *other;
+
+  g_return_if_fail (GLADE_IS_COMMAND_DOMAIN (this_cmd) &&
+                    GLADE_IS_COMMAND_DOMAIN (other_cmd));
+
+  this = GLADE_COMMAND_DOMAIN (this_cmd);
+  other = GLADE_COMMAND_DOMAIN (other_cmd);
+
+  g_free (this->new_domain);
+  this->new_domain = g_strdup (other->new_domain);
+
+  g_free (this_cmd->priv->description);
+  this_cmd->priv->description =
+    g_strdup_printf (_("Setting translation domain to '%s'"), 
+		     this->new_domain);
+}
+
+/**
+ * glade_command_set_project_domain:
+ * @project: A #GladeProject
+ * @domain: The translation domain for @project
+ *
+ * Sets @domain as the translation domain for @project.
+ */
+void
+glade_command_set_project_domain  (GladeProject *project,
+				   const gchar  *domain)
+{
+  GladeCommandDomain *me;
+  const gchar *old_domain;
+
+  g_return_if_fail (GLADE_IS_PROJECT (project));
+
+  old_domain = glade_project_get_translation_domain (project);
+
+  /* Do nothing if nothing has changed */
+  if (g_strcmp0 (domain, old_domain) == 0)
+    return;
+
+  /* load up the command */
+  me = g_object_new (GLADE_COMMAND_DOMAIN_TYPE, NULL);
+  GLADE_COMMAND (me)->priv->project = project;
+
+  me->new_domain = g_strdup (domain);
+  me->old_domain = g_strdup (old_domain);
+
+  GLADE_COMMAND (me)->priv->description =
+    g_strdup_printf (_("Setting translation domain to '%s'"), 
+		     me->new_domain);
+
+  glade_command_check_group (GLADE_COMMAND (me));
+
+  /* execute the command and push it on the stack if successful 
+   * this sets the actual policy
+   */
+  if (glade_command_domain_execute (GLADE_COMMAND (me)))
+    glade_project_push_undo (GLADE_COMMAND (me)->priv->project, GLADE_COMMAND (me));
+  else
+    g_object_unref (G_OBJECT (me));
+}
+
+/******************************************************************************
+ * 
+ * This command sets the template object in a GtkBuilder file
+ * 
+ *****************************************************************************/
+
+typedef struct
+{
+  GladeCommand parent;
+  GladeWidget *old_template;
+  GladeWidget *new_template;
+} GladeCommandTemplate;
+
+
+GLADE_MAKE_COMMAND (GladeCommandTemplate, glade_command_template);
+#define GLADE_COMMAND_TEMPLATE_TYPE			(glade_command_template_get_type ())
+#define GLADE_COMMAND_TEMPLATE(o)	  		(G_TYPE_CHECK_INSTANCE_CAST ((o), GLADE_COMMAND_TEMPLATE_TYPE, GladeCommandTemplate))
+#define GLADE_COMMAND_TEMPLATE_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), GLADE_COMMAND_TEMPLATE_TYPE, GladeCommandTemplateClass))
+#define GLADE_IS_COMMAND_TEMPLATE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), GLADE_COMMAND_TEMPLATE_TYPE))
+#define GLADE_IS_COMMAND_TEMPLATE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), GLADE_COMMAND_TEMPLATE_TYPE))
+
+static gboolean
+glade_command_template_execute (GladeCommand *cmd)
+{
+  GladeCommandTemplate *me = (GladeCommandTemplate *) cmd;
+
+  glade_project_set_template (cmd->priv->project, me->new_template);
+
+  return TRUE;
+}
+
+static gboolean
+glade_command_template_undo (GladeCommand *cmd)
+{
+  GladeCommandTemplate *me = (GladeCommandTemplate *) cmd;
+
+  glade_project_set_template (cmd->priv->project, me->old_template);
+
+  return TRUE;
+}
+
+static void
+glade_command_template_finalize (GObject *obj)
+{
+  GladeCommandTemplate *me = (GladeCommandTemplate *) obj;
+
+  if (me->new_template)
+    g_object_unref (me->new_template);
+
+  if (me->old_template)
+    g_object_unref (me->old_template);
+
+  glade_command_finalize (obj);
+}
+
+static gboolean
+glade_command_template_unifies (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandTemplate *me;
+
+  /* Do we unify with self ? */
+  if (!other_cmd)
+    {
+      if (GLADE_IS_COMMAND_TEMPLATE (this_cmd))
+        {
+          me = (GladeCommandTemplate *) this_cmd;
+
+	  return me->old_template == me->new_template;
+        }
+      return FALSE;
+    }
+
+  if (GLADE_IS_COMMAND_TEMPLATE (this_cmd) &&
+      GLADE_IS_COMMAND_TEMPLATE (other_cmd))
+    {
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+glade_command_template_collapse (GladeCommand *this_cmd, GladeCommand *other_cmd)
+{
+  GladeCommandTemplate *this;
+  GladeCommandTemplate *other;
+
+  g_return_if_fail (GLADE_IS_COMMAND_TEMPLATE (this_cmd) &&
+                    GLADE_IS_COMMAND_TEMPLATE (other_cmd));
+
+  this = GLADE_COMMAND_TEMPLATE (this_cmd);
+  other = GLADE_COMMAND_TEMPLATE (other_cmd);
+
+  if (this->new_template)
+    g_object_unref (this->new_template);
+
+  this->new_template = other->new_template;
+
+  if (this->new_template)
+    g_object_ref (this->new_template);
+
+  if (this->new_template == NULL && this->old_template != NULL)
+    {
+      g_free (this_cmd->priv->description);
+      this_cmd->priv->description =
+	g_strdup_printf (_("Unsetting widget '%s' as template"), 
+			 glade_widget_get_name (this->old_template));
+    }
+  else if (this->new_template != NULL)
+    {
+      g_free (this_cmd->priv->description);
+      this_cmd->priv->description =
+	g_strdup_printf (_("Setting widget '%s' as template"), 
+			 glade_widget_get_name (this->new_template));
+    }
+}
+
+/**
+ * glade_command_set_project_template:
+ * @project: A #GladeProject
+ * @widget: The #GladeWidget to make template
+ *
+ * Sets @widget to be the template widget in @project.
+ */
+void
+glade_command_set_project_template (GladeProject *project,
+				    GladeWidget  *widget)
+{
+  GladeCommandTemplate *me;
+  GladeWidget *old_template;
+
+  g_return_if_fail (GLADE_IS_PROJECT (project));
+  g_return_if_fail (widget == NULL || GLADE_IS_WIDGET (widget));
+
+  old_template = glade_project_get_template (project);
+
+  if (widget == old_template)
+    {
+      /* Just do nothing if there's nothing to do */
+      return;
+    }
+
+  /* load up the command */
+  me = g_object_new (GLADE_COMMAND_TEMPLATE_TYPE, NULL);
+  GLADE_COMMAND (me)->priv->project = project;
+
+  if (old_template)
+    me->old_template = g_object_ref (old_template);
+
+  if (widget)
+    {
+      me->new_template = g_object_ref (widget);
+
+      GLADE_COMMAND (me)->priv->description =
+	g_strdup_printf (_("Setting widget '%s' as template"), 
+			 glade_widget_get_name (widget));
+    }
+  else
+    GLADE_COMMAND (me)->priv->description =
+      g_strdup_printf (_("Unsetting widget '%s' as template"), 
+		       glade_widget_get_name (me->old_template));
+
+  glade_command_check_group (GLADE_COMMAND (me));
+
+  /* execute the command and push it on the stack if successful 
+   * this sets the actual policy
+   */
+  if (glade_command_template_execute (GLADE_COMMAND (me)))
+    glade_project_push_undo (GLADE_COMMAND (me)->priv->project, GLADE_COMMAND (me));
+  else
+    g_object_unref (G_OBJECT (me));
+}
+
