@@ -370,9 +370,6 @@ glade_design_view_init (GladeDesignView *view)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW
                                   (view->priv->scrolled_window),
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW
-                                       (view->priv->scrolled_window),
-                                       GTK_SHADOW_IN);
 
   viewport = gtk_viewport_new (NULL, NULL);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
@@ -544,7 +541,7 @@ glade_design_view_drag_motion (GtkWidget *widget,
       if (priv->drag_source &&
           (priv->drag_source == child || gtk_widget_is_ancestor (child, priv->drag_source) ||
            (!GLADE_IS_PLACEHOLDER (child) && 
-            !GTK_IS_FIXED (child) &&
+            !GTK_IS_FIXED (child) && !GTK_IS_LAYOUT (child) &&
             (glade_widget_get_from_gobject (child) ||
              ((gwidget = glade_widget_get_from_gobject (priv->drag_source)) &&
              !glade_widget_get_parent (gwidget)
@@ -554,7 +551,7 @@ glade_design_view_drag_motion (GtkWidget *widget,
 
       if (priv->drag_adaptor &&
           ((GLADE_IS_PLACEHOLDER (child) && GWA_IS_TOPLEVEL (priv->drag_adaptor)) ||
-           (!GLADE_IS_PLACEHOLDER (child) && !GTK_IS_FIXED (child) &&
+           (!GLADE_IS_PLACEHOLDER (child) && !GTK_IS_FIXED (child) && !GTK_IS_LAYOUT (child) &&
             glade_widget_get_from_gobject (child))))
         drag_action = 0;
     }
@@ -650,35 +647,6 @@ glade_design_view_drag_data_received (GtkWidget        *widget,
                     GLADE_DESIGN_VIEW (widget));
 }
 
-
-static void
-glade_design_view_fixed_move (GladeWidget *gsource,
-                              GtkWidget *widget,
-                              GdkDragContext *context,
-                              GtkWidget *child,
-                              gint x,
-                              gint y)
-{
-  GladeProperty *prop_x, *prop_y;
-  gint dx, dy, hx, hy;
-  GtkWidget *layout;
-
-  gtk_widget_translate_coordinates (widget, child, x, y, &dx, &dy);
-
-  prop_x = glade_widget_get_pack_property (gsource, "x");
-  prop_y = glade_widget_get_pack_property (gsource, "y");
-
-  layout = gtk_drag_get_source_widget (context);
-
-  if (layout && GLADE_IS_DESIGN_LAYOUT (layout))
-    _glade_design_layout_get_hot_point (GLADE_DESIGN_LAYOUT (layout), &hx, &hy);
-  else
-    hx = hy = 0;
-
-  glade_command_set_property (prop_x, dx - hx);
-  glade_command_set_property (prop_y, dy - hy);
-}
-
 static gboolean
 glade_design_view_drag_drop (GtkWidget       *widget,
                              GdkDragContext  *context,
@@ -705,16 +673,11 @@ glade_design_view_drag_drop (GtkWidget       *widget,
           if (parent != gsource)
             glade_command_dnd (&widgets, parent, placeholder);
         }
-      else if (GTK_IS_FIXED (child))
+      else if (GTK_IS_FIXED (child) || GTK_IS_LAYOUT (child))
         {
           GladeWidget *parent = glade_widget_get_from_gobject (child);
-          
-          glade_command_push_group ("Drag and Drop");
-          if (parent != glade_widget_get_parent (gsource))
-            glade_command_dnd (&widgets, parent, NULL);
-          
-          glade_design_view_fixed_move (gsource, widget, context, child, x, y);
-          glade_command_pop_group ();
+
+	  glade_command_dnd (&widgets, parent, NULL);
         }
       else if (!glade_widget_get_from_gobject (child))
         glade_command_dnd (&widgets, NULL, NULL);
@@ -730,22 +693,14 @@ glade_design_view_drag_drop (GtkWidget       *widget,
                                 placeholder, 
                                 priv->project);
         }
-      else if (GTK_IS_FIXED (child))
+      else if (GTK_IS_FIXED (child) || GTK_IS_LAYOUT (child))
         {
           GladeWidget *parent = glade_widget_get_from_gobject (child);
 
           if (parent)
-            {
-              GladeWidget *gsource;
-              glade_command_push_group ("Drag and Drop");
-
-              gsource = glade_command_create (priv->drag_adaptor,
-                                              parent, NULL,
-                                              priv->project);
-
-              glade_design_view_fixed_move (gsource, widget, context, child, x, y);
-              glade_command_pop_group ();
-            }
+	    glade_command_create (priv->drag_adaptor,
+				  parent, NULL,
+				  priv->project);
         }
       else
         {
