@@ -20,18 +20,46 @@
  */
 
 #include <config.h>
-#include <gladeui/glade.h>
 #include <glib/gi18n-lib.h>
+#include "glade.h"
+#include "gladeui-enum-types.h"
 
 #include "glade-editor-table.h"
 
+
+#define BLOCK_NAME_ENTRY_CB(table)					\
+	do { if (table->priv->name_entry)					\
+			g_signal_handlers_block_by_func (G_OBJECT (table->priv->name_entry), \
+							 G_CALLBACK (widget_name_edited), table); \
+	} while (0);
+
+#define UNBLOCK_NAME_ENTRY_CB(table)					\
+	do { if (table->priv->name_entry)					\
+			g_signal_handlers_unblock_by_func (G_OBJECT (table->priv->name_entry), \
+							   G_CALLBACK (widget_name_edited), table); \
+	} while (0);
+
+
+
 static void glade_editor_table_init          (GladeEditorTable * self);
 static void glade_editor_table_class_init    (GladeEditorTableClass * klass);
-static void glade_editor_table_dispose       (GObject * object);
+
+static void glade_editor_table_dispose       (GObject         *object);
+static void glade_editor_table_set_property  (GObject         *object,
+					      guint            prop_id,
+					      const GValue    *value,
+					      GParamSpec      *pspec);
+
 static void glade_editor_table_editable_init (GladeEditableIface * iface);
 static void glade_editor_table_realize       (GtkWidget * widget);
 static void glade_editor_table_grab_focus    (GtkWidget * widget);
 
+static void append_name_field (GladeEditorTable   *table);
+static void append_items      (GladeEditorTable   *table,
+			       GladeWidgetAdaptor *adaptor,
+			       GladeEditorPageType type);
+
+
 struct _GladeEditorTablePrivate
 {
   GladeWidgetAdaptor *adaptor; /* The GladeWidgetAdaptor this
@@ -49,6 +77,8 @@ struct _GladeEditorTablePrivate
 			  * entry which will not be created from a
 			  * GladeProperty but rather from code.
 			  */
+  GtkWidget *composite_check; /* A pointer to the composite check button */
+  GtkWidget *name_field; /* A box containing the name entry and composite check */
 
   GList *properties; /* A list of GladeEditorPropery items.
 		      * For each row in the gtk_table, there is a
@@ -62,24 +92,15 @@ struct _GladeEditorTablePrivate
   gint rows;
 };
 
+enum {
+  PROP_0,
+  PROP_PAGE_TYPE,
+};
+
 G_DEFINE_TYPE_WITH_CODE (GladeEditorTable, glade_editor_table, GTK_TYPE_GRID,
                          G_IMPLEMENT_INTERFACE (GLADE_TYPE_EDITABLE,
                                                 glade_editor_table_editable_init));
 
-
-#define BLOCK_NAME_ENTRY_CB(table)					\
-	do { if (table->priv->name_entry)					\
-			g_signal_handlers_block_by_func (G_OBJECT (table->priv->name_entry), \
-							 G_CALLBACK (widget_name_edited), table); \
-	} while (0);
-
-#define UNBLOCK_NAME_ENTRY_CB(table)					\
-	do { if (table->priv->name_entry)					\
-			g_signal_handlers_unblock_by_func (G_OBJECT (table->priv->name_entry), \
-							   G_CALLBACK (widget_name_edited), table); \
-	} while (0);
-
-
 static void
 glade_editor_table_class_init (GladeEditorTableClass * klass)
 {
@@ -87,9 +108,18 @@ glade_editor_table_class_init (GladeEditorTableClass * klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->dispose = glade_editor_table_dispose;
+  object_class->set_property = glade_editor_table_set_property;
+
   widget_class->realize = glade_editor_table_realize;
   widget_class->grab_focus = glade_editor_table_grab_focus;
 
+  g_object_class_install_property
+      (object_class, PROP_PAGE_TYPE,
+       g_param_spec_enum ("page-type", _("Page Type"),
+			  _("The editor page type to create this GladeEditorTable for"),
+			  GLADE_TYPE_EDITOR_PAGE_TYPE, GLADE_PAGE_GENERAL,
+			  G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
   g_type_class_add_private (klass, sizeof (GladeEditorTablePrivate));
 }
 
@@ -103,7 +133,8 @@ glade_editor_table_init (GladeEditorTable * self)
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (self),
                                   GTK_ORIENTATION_VERTICAL);
-  gtk_grid_set_row_spacing (GTK_GRID (self), 4);
+  gtk_grid_set_row_spacing (GTK_GRID (self), 2);
+  gtk_grid_set_column_spacing (GTK_GRID (self), 6);
 }
 
 static void
@@ -123,6 +154,24 @@ glade_editor_table_dispose (GObject * object)
   G_OBJECT_CLASS (glade_editor_table_parent_class)->dispose (object);
 }
 
+static void
+glade_editor_table_set_property (GObject         *object,
+				 guint            prop_id,
+				 const GValue    *value,
+				 GParamSpec      *pspec)
+{
+  GladeEditorTable *table = GLADE_EDITOR_TABLE (object);
+
+  switch (prop_id)
+    {
+    case PROP_PAGE_TYPE:
+      table->priv->type = g_value_get_enum (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
 
 static void
 glade_editor_table_realize (GtkWidget * widget)
@@ -181,8 +230,33 @@ widget_name_edited (GtkWidget * editable, GladeEditorTable * table)
 }
 
 static void
-widget_name_changed (GladeWidget * widget,
-                     GParamSpec * pspec, GladeEditorTable * table)
+widget_composite_toggled (GtkToggleButton  *composite_check,
+			  GladeEditorTable *table)
+{
+  GladeProject *project;
+
+  if (table->priv->loaded_widget == NULL)
+    {
+      g_warning ("Name entry edited with no loaded widget in editor %p!\n",
+                 table);
+      return;
+    }
+
+  project = glade_widget_get_project (table->priv->loaded_widget);
+
+  if (project)
+    {
+      if (gtk_toggle_button_get_active (composite_check))
+	glade_command_set_project_template (project, table->priv->loaded_widget);
+      else
+	glade_command_set_project_template (project, NULL);
+    }
+}
+
+static void
+widget_name_changed (GladeWidget      *widget,
+                     GParamSpec       *pspec,
+		     GladeEditorTable *table)
 {
   if (!gtk_widget_get_mapped (GTK_WIDGET (table)))
     return;
@@ -194,7 +268,30 @@ widget_name_changed (GladeWidget * widget,
                           glade_widget_get_name (table->priv->loaded_widget));
       UNBLOCK_NAME_ENTRY_CB (table);
     }
+}
 
+static void
+widget_composite_changed (GladeWidget      *widget,
+			  GParamSpec       *pspec,
+			  GladeEditorTable *table)
+{
+  if (!gtk_widget_get_mapped (GTK_WIDGET (table)))
+    return;
+
+  if (table->priv->name_label)
+    gtk_label_set_text (GTK_LABEL (table->priv->name_label),
+			glade_widget_get_is_composite (table->priv->loaded_widget) ?
+			_("Class Name:") : _("ID:"));
+
+  if (table->priv->composite_check)
+    {
+      g_signal_handlers_block_by_func (G_OBJECT (table->priv->composite_check),
+				       G_CALLBACK (widget_composite_toggled), table);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (table->priv->composite_check),
+				    glade_widget_get_is_composite (table->priv->loaded_widget));
+      g_signal_handlers_unblock_by_func (G_OBJECT (table->priv->composite_check),
+					 G_CALLBACK (widget_composite_toggled), table);
+    }
 }
 
 static void
@@ -213,6 +310,17 @@ glade_editor_table_load (GladeEditable * editable, GladeWidget * widget)
   GladeEditorProperty *property;
   GList *list;
 
+  /* Setup the table the first time the widget is loaded */
+  if (widget && table->priv->adaptor == NULL)
+    {
+      table->priv->adaptor = glade_widget_get_adaptor (widget);
+
+      if (table->priv->type == GLADE_PAGE_GENERAL)
+	append_name_field (table);
+
+      append_items (table, table->priv->adaptor, table->priv->type);
+    }
+
   /* abort mission */
   if (table->priv->loaded_widget == widget)
     return;
@@ -222,6 +330,9 @@ glade_editor_table_load (GladeEditable * editable, GladeWidget * widget)
       g_signal_handlers_disconnect_by_func (G_OBJECT (table->priv->loaded_widget),
                                             G_CALLBACK (widget_name_changed),
                                             table);
+      g_signal_handlers_disconnect_by_func (G_OBJECT (table->priv->loaded_widget),
+                                            G_CALLBACK (widget_composite_changed),
+                                            table);
 
       /* The widget could die unexpectedly... */
       g_object_weak_unref (G_OBJECT (table->priv->loaded_widget),
@@ -237,14 +348,30 @@ glade_editor_table_load (GladeEditable * editable, GladeWidget * widget)
       g_signal_connect (G_OBJECT (table->priv->loaded_widget), "notify::name",
                         G_CALLBACK (widget_name_changed), table);
 
+      g_signal_connect (G_OBJECT (table->priv->loaded_widget), "notify::composite",
+                        G_CALLBACK (widget_composite_changed), table);
+
       /* The widget could die unexpectedly... */
       g_object_weak_ref (G_OBJECT (table->priv->loaded_widget),
                          (GWeakNotify) widget_finalized, table);
 
+      if (table->priv->composite_check)
+	{
+	  GObject *object = glade_widget_get_object (table->priv->loaded_widget);
+
+	  if (GTK_IS_WIDGET (object) && 
+	      glade_widget_get_parent (table->priv->loaded_widget) == NULL)
+	    gtk_widget_show (table->priv->composite_check);
+	  else
+	    gtk_widget_hide (table->priv->composite_check);
+	}
+
       if (table->priv->name_entry)
         gtk_entry_set_text (GTK_ENTRY (table->priv->name_entry), 
 			    glade_widget_get_name (widget));
 
+      if (table->priv->name_label)
+	widget_composite_changed (widget, NULL, table);
     }
   else if (table->priv->name_entry)
     gtk_entry_set_text (GTK_ENTRY (table->priv->name_entry), "");
@@ -269,12 +396,12 @@ glade_editor_table_set_show_name (GladeEditable * editable, gboolean show_name)
       if (show_name)
         {
           gtk_widget_show (table->priv->name_label);
-          gtk_widget_show (table->priv->name_entry);
+          gtk_widget_show (table->priv->name_field);
         }
       else
         {
           gtk_widget_hide (table->priv->name_label);
-          gtk_widget_hide (table->priv->name_entry);
+          gtk_widget_hide (table->priv->name_field);
         }
     }
 }
@@ -301,11 +428,21 @@ property_class_comp (gconstpointer a, gconstpointer b)
 {
   GladePropertyClass *ca = (GladePropertyClass *)a, *cb = (GladePropertyClass *)b;
   GParamSpec *pa, *pb;
+  const gchar *name_a, *name_b;
 
   pa = glade_property_class_get_pspec (ca);
   pb = glade_property_class_get_pspec (cb);
 
-  if (pa->owner_type == pb->owner_type)
+  name_a = glade_property_class_id (ca);
+  name_b = glade_property_class_id (cb);
+
+  /* Special case for the 'name' property, it *always* comes first. */
+  if (strcmp (name_a, "name") == 0)
+    return -1;
+  else if (strcmp (name_b, "name") == 0)
+    return 1;
+  /* Properties of the same class are sorted in the same level */
+  else if (pa->owner_type == pb->owner_type)
     {
       gdouble result = glade_property_class_weight (ca) - glade_property_class_weight (cb);
       /* Avoid cast to int */
@@ -316,6 +453,7 @@ property_class_comp (gconstpointer a, gconstpointer b)
       else
         return 0;
     }
+  /* Group properties by thier class hierarchy */
   else
     {
       if (g_type_is_a (pa->owner_type, pb->owner_type))
@@ -359,6 +497,7 @@ append_item (GladeEditorTable * table,
              GladePropertyClass * klass, gboolean from_query_dialog)
 {
   GladeEditorProperty *property;
+  GtkWidget *label;
 
   if (!(property = glade_widget_adaptor_create_eprop
         (glade_property_class_get_adaptor (klass), klass, from_query_dialog == FALSE)))
@@ -372,7 +511,10 @@ append_item (GladeEditorTable * table,
   gtk_widget_show (GTK_WIDGET (property));
   gtk_widget_show_all (glade_editor_property_get_item_label (property));
 
-  glade_editor_table_attach (table, glade_editor_property_get_item_label (property), 0, table->priv->rows);
+  label = glade_editor_property_get_item_label (property);
+  gtk_widget_set_hexpand (label, FALSE);
+
+  glade_editor_table_attach (table, label, 0, table->priv->rows);
   glade_editor_table_attach (table, GTK_WIDGET (property), 1, table->priv->rows);
 
   table->priv->rows++;
@@ -380,7 +522,6 @@ append_item (GladeEditorTable * table,
   return property;
 }
 
-
 static void
 append_items (GladeEditorTable * table,
               GladeWidgetAdaptor * adaptor, GladeEditorPageType type)
@@ -405,17 +546,25 @@ append_items (GladeEditorTable * table,
 static void
 append_name_field (GladeEditorTable * table)
 {
-  gchar *text = _("The Object's name");
+  gchar *text = _("The object's unique identifier");
 
-  /* Name */
-  table->priv->name_label = gtk_label_new (_("Name:"));
+  /* translators: The unique identifier of an object in the project */
+  table->priv->name_label = gtk_label_new (_("ID:"));
   gtk_misc_set_alignment (GTK_MISC (table->priv->name_label), 0.0, 0.5);
   gtk_widget_show (table->priv->name_label);
   gtk_widget_set_no_show_all (table->priv->name_label, TRUE);
 
+  table->priv->name_field = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_widget_set_no_show_all (table->priv->name_field, TRUE);
+  gtk_widget_show (table->priv->name_field);
+
+  table->priv->composite_check = gtk_check_button_new_with_label (_("Composite"));
+  gtk_widget_set_hexpand (table->priv->composite_check, FALSE);
+  gtk_widget_set_tooltip_text (table->priv->composite_check, _("Whether this widget is a composite template"));
+  gtk_widget_set_no_show_all (table->priv->composite_check, TRUE);
+
   table->priv->name_entry = gtk_entry_new ();
   gtk_widget_show (table->priv->name_entry);
-  gtk_widget_set_no_show_all (table->priv->name_entry, TRUE);
 
   gtk_widget_set_tooltip_text (table->priv->name_label, text);
   gtk_widget_set_tooltip_text (table->priv->name_entry, text);
@@ -424,9 +573,14 @@ append_name_field (GladeEditorTable * table)
                     G_CALLBACK (widget_name_edited), table);
   g_signal_connect (G_OBJECT (table->priv->name_entry), "changed",
                     G_CALLBACK (widget_name_edited), table);
+  g_signal_connect (G_OBJECT (table->priv->composite_check), "toggled",
+                    G_CALLBACK (widget_composite_toggled), table);
+
+  gtk_box_pack_start (GTK_BOX (table->priv->name_field), table->priv->name_entry, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (table->priv->name_field), table->priv->composite_check, FALSE, FALSE, 0);
 
   glade_editor_table_attach (table, table->priv->name_label, 0, table->priv->rows);
-  glade_editor_table_attach (table, table->priv->name_entry, 1, table->priv->rows);
+  glade_editor_table_attach (table, table->priv->name_field, 1, table->priv->rows);
 
   table->priv->rows++;
 }
@@ -448,16 +602,13 @@ glade_editor_table_new (GladeWidgetAdaptor * adaptor, GladeEditorPageType type)
 
   g_return_val_if_fail (GLADE_IS_WIDGET_ADAPTOR (adaptor), NULL);
 
-  table = g_object_new (GLADE_TYPE_EDITOR_TABLE, NULL);
+  table = g_object_new (GLADE_TYPE_EDITOR_TABLE, "page-type", type, NULL);
   table->priv->adaptor = adaptor;
-  table->priv->type = type;
 
   if (type == GLADE_PAGE_GENERAL)
     append_name_field (table);
 
   append_items (table, adaptor, type);
 
-  gtk_widget_show (GTK_WIDGET (table));
-
   return GTK_WIDGET (table);
 }
