@@ -501,8 +501,7 @@ glade_eprop_accel_accum_accelerators (GtkTreeModel * model,
 
 
 static void
-glade_eprop_accel_show_dialog (GtkWidget * dialog_button,
-                               GladeEditorProperty * eprop)
+glade_eprop_accel_show_dialog (GladeEditorProperty *eprop)
 {
   GladeEPropAccel *eprop_accel = GLADE_EPROP_ACCEL (eprop);
   GtkWidget *dialog, *parent, *vbox, *sw, *tree_view;
@@ -590,23 +589,17 @@ static GtkWidget *
 glade_eprop_accel_create_input (GladeEditorProperty * eprop)
 {
   GladeEPropAccel *eprop_accel = GLADE_EPROP_ACCEL (eprop);
-  GtkWidget *hbox;
-  GtkWidget *button;
 
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   eprop_accel->entry = gtk_entry_new ();
+  gtk_widget_set_valign (eprop_accel->entry, GTK_ALIGN_CENTER);
   gtk_editable_set_editable (GTK_EDITABLE (eprop_accel->entry), FALSE);
-  gtk_widget_show (eprop_accel->entry);
-  gtk_box_pack_start (GTK_BOX (hbox), eprop_accel->entry, TRUE, TRUE, 0);
+  gtk_entry_set_icon_from_stock (GTK_ENTRY (eprop_accel->entry), 
+                                 GTK_ENTRY_ICON_SECONDARY,
+                                 GTK_STOCK_EDIT);
+  g_signal_connect_swapped (eprop_accel->entry, "icon-release",
+                            G_CALLBACK (glade_eprop_accel_show_dialog), eprop);
 
-  button = gtk_button_new_with_label ("...");
-  gtk_widget_show (button);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
-  g_signal_connect (G_OBJECT (button), "clicked",
-                    G_CALLBACK (glade_eprop_accel_show_dialog), eprop);
-
-  return hbox;
+  return eprop_accel->entry;
 }
 
 
